@@ -23,14 +23,34 @@ print.restriktor <- function(x, digits = max(3, getOption("digits") - 2), ...) {
     }
   } else if (inherits(x,"conLM")) {
     cat("restricted linear model:\n\n")
+  } else if (inherits(x,"conMLM")) {
+    cat("restricted multivariate linear model:\n\n")
   }
   
-  if (length(coef(x)) > 0L) {
+  coef <- coef(x)
+  ny <- ncol(coef(x$model.org))
+  if (!is.null(ny) && ny > 1L) {
+    ynames <- colnames(coef(x))
+    if (is.null(ynames)) {
+      lhs <- x$model.org$terms[[2L]]
+      if (mode(lhs) == "call" && lhs[[1L]] == "cbind") {
+        ynames <- as.character(lhs)[-1L]
+      } else {
+        ynames <- paste0("Y", seq_len(ny))
+      }
+    }
+    ind <- ynames == ""
+    if (any(ind)) 
+      ynames[ind] <- paste0("Y", seq_len(ny))[ind]
+    colnames(coef) <- ynames
+  }
+  
+  if (length(coef) > 0L) { 
     cat("Coefficients:\n")
-    print(coef(x), digits = digits, scientific = FALSE, print.gap = 2L,
+    print(coef, digits = digits, scientific = FALSE, print.gap = 2L,
           quote = FALSE)
   } else {
     cat("No coefficients\n")
-  }  
+  }   
   invisible(x)
 }
